@@ -6,14 +6,13 @@
 """
 import datetime
 import hashlib
+import ipaddress
 import logging
 import re
-from typing import List
 from typing import Optional
-from typing import Set
-from typing import Tuple
 from typing import TYPE_CHECKING
 from typing import Union
+import warnings
 
 from cryptography import x509
 from cryptography.exceptions import InvalidSignature
@@ -100,8 +99,10 @@ def generate_key(key_size: int, key_dir: Optional[str], key_type: str = "rsa",
     return util.Key(key_path, key_pem)
 
 
-def generate_csr(privkey: util.Key, names: Union[List[str], Set[str]], path: Optional[str],
-                 must_staple: bool = False, strict_permissions: bool = True) -> util.CSR:
+def generate_csr(privkey: util.Key, names: Union[list[str], set[str]], path: Optional[str],
+                 must_staple: bool = False, strict_permissions: bool = True,
+                 ipaddrs: list[ipaddress.IPv4Address | ipaddress.IPv6Address] | None = None,
+                 ) -> util.CSR:
     """Initialize a CSR with the given private key.
 
     :param privkey: Key to include in the CSR
@@ -117,7 +118,7 @@ def generate_csr(privkey: util.Key, names: Union[List[str], Set[str]], path: Opt
 
     """
     csr_pem = acme_crypto_util.make_csr(
-        privkey.pem, names, must_staple=must_staple)
+        privkey.pem, names, must_staple=must_staple, ipaddrs=ipaddrs)
 
     # Save CSR, if requested
     csr_filename = None
@@ -171,9 +172,36 @@ def csr_matches_pubkey(csr: bytes, privkey: bytes) -> bool:
     return req.is_signature_valid and req.public_key() == pkey.public_key()
 
 
+def read_csr_file(
+    csrfile: str, data: bytes
+) -> util.CSR:
+    """Reads a CSR file, which can be either PEM or DER, and returns a
+    `certbot.util.CSR` object.
+
+    :param str csrfile: CSR filename
+    :param bytes data: contents of the CSR file
+
+    :returns: object representing the CSR
+    :rtype: util.CSR
+
+    """
+    try:
+        # Try to parse as DER first, then fall back to PEM.
+        csr = x509.load_der_x509_csr(data)
+    except ValueError:
+        try:
+            csr = x509.load_pem_x509_csr(data)
+        except ValueError:
+            raise errors.Error("Failed to parse CSR file: {0}".format(csrfile))
+
+    # Internally we always use PEM, so re-encode as PEM before returning.
+    data_pem = csr.public_bytes(serialization.Encoding.PEM)
+    return util.CSR(file=csrfile, data=data_pem, form="pem")
+
+
 def import_csr_file(
     csrfile: str, data: bytes
-) -> Tuple[acme_crypto_util.Format, util.CSR, List[str]]:
+) -> tuple['acme_crypto_util.Format', util.CSR, list[str]]:
     """Import a CSR file, which can be either PEM or DER.
 
     :param str csrfile: CSR filename
@@ -185,6 +213,9 @@ def import_csr_file(
     :rtype: tuple
 
     """
+    warnings.warn("certbot.crypto_util.import_csr_file is deprecated and "
+        "will be removed in the next major release. Please use "
+        "certbot.crypto_util.read_csr_file instead.", DeprecationWarning)
     try:
         # Try to parse as DER first, then fall back to PEM.
         csr = x509.load_der_x509_csr(data)
@@ -197,11 +228,13 @@ def import_csr_file(
     domains = acme_crypto_util.get_names_from_subject_and_extensions(csr.subject, csr.extensions)
     # Internally we always use PEM, so re-encode as PEM before returning.
     data_pem = csr.public_bytes(serialization.Encoding.PEM)
-    return (
-        acme_crypto_util.Format.PEM,
-        util.CSR(file=csrfile, data=data_pem, form="pem"),
-        domains,
-    )
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", "acme.crypto_util.Format is deprecated")
+        return (
+            acme_crypto_util.Format.PEM,
+            util.CSR(file=csrfile, data=data_pem, form="pem"),
+            domains,
+        )
 
 
 def make_key(bits: int = 2048, key_type: str = "rsa",
@@ -348,7 +381,7 @@ def verify_cert_matches_priv_key(cert_path: str, key_path: str) -> None:
     :raises errors.Error: If they don't match.
     """
     try:
-        context = SSL.Context(SSL.SSLv23_METHOD)
+        context = SSL.Context(SSL.TLS_METHOD)
         context.use_certificate_file(cert_path)
         context.use_privatekey_file(key_path)
         context.check_privatekey()
@@ -389,8 +422,8 @@ def verify_fullchain(renewable_cert: interfaces.RenewableCert) -> None:
 
 
 def get_sans_from_cert(
-    cert: bytes, typ: Union[acme_crypto_util.Format, int] = acme_crypto_util.Format.PEM
-) -> List[str]:
+    cert: bytes, typ: 'acme_crypto_util.Format | int | None' = None
+) -> list[str]:
     """Get a list of Subject Alternative Names from a certificate.
 
     :param str cert: Certificate (encoded).
@@ -400,12 +433,18 @@ def get_sans_from_cert(
     :rtype: list
 
     """
-    typ = acme_crypto_util.Format(typ)
-    if typ == acme_crypto_util.Format.PEM:
-        x509_cert = x509.load_pem_x509_certificate(cert)
-    else:
-        assert typ == acme_crypto_util.Format.DER
-        x509_cert = x509.load_der_x509_certificate(cert)
+    warnings.warn("get_sans_from_cert is deprecated and will be removed in the next "
+        "major release.", DeprecationWarning)
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", "acme.crypto_util.Format is deprecated")
+        if typ is None:
+            typ = acme_crypto_util.Format.PEM
+        typ = acme_crypto_util.Format(typ)
+        if typ == acme_crypto_util.Format.PEM:
+            x509_cert = x509.load_pem_x509_certificate(cert)
+        else:
+            assert typ == acme_crypto_util.Format.DER
+            x509_cert = x509.load_der_x509_certificate(cert)
 
     try:
         san_ext = x509_cert.extensions.get_extension_for_class(
@@ -418,8 +457,8 @@ def get_sans_from_cert(
 
 
 def get_names_from_cert(
-    cert: bytes, typ: Union[acme_crypto_util.Format, int] = acme_crypto_util.Format.PEM
-) -> List[str]:
+    cert: bytes, typ: 'acme_crypto_util.Format | int | None' = None
+) -> list[str]:
     """Get a list of domains from a cert, including the CN if it is set.
 
     :param str cert: Certificate (encoded).
@@ -429,20 +468,26 @@ def get_names_from_cert(
     :rtype: list
 
     """
-    typ = acme_crypto_util.Format(typ)
-    if typ == acme_crypto_util.Format.PEM:
-        x509_cert = x509.load_pem_x509_certificate(cert)
-    else:
-        assert typ == acme_crypto_util.Format.DER
-        x509_cert = x509.load_der_x509_certificate(cert)
+    warnings.warn("get_names_from_cert is deprecated and will be removed in the next "
+        "major release.", DeprecationWarning)
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", "acme.crypto_util.Format is deprecated")
+        if typ is None:
+            typ = acme_crypto_util.Format.PEM
+        typ = acme_crypto_util.Format(typ)
+        if typ == acme_crypto_util.Format.PEM:
+            x509_cert = x509.load_pem_x509_certificate(cert)
+        else:
+            assert typ == acme_crypto_util.Format.DER
+            x509_cert = x509.load_der_x509_certificate(cert)
     return acme_crypto_util.get_names_from_subject_and_extensions(
         x509_cert.subject, x509_cert.extensions
     )
 
 
 def get_names_from_req(
-    csr: bytes, typ: Union[acme_crypto_util.Format, int] = acme_crypto_util.Format.PEM
-) -> List[str]:
+    csr: bytes, typ: 'acme_crypto_util.Format | int | None' = None
+) -> list[str]:
     """Get a list of domains from a CSR, including the CN if it is set.
 
     :param str csr: CSR (encoded).
@@ -451,12 +496,18 @@ def get_names_from_req(
     :rtype: list
 
     """
-    typ = acme_crypto_util.Format(typ)
-    if typ == acme_crypto_util.Format.PEM:
-        x509_req = x509.load_pem_x509_csr(csr)
-    else:
-        assert typ == acme_crypto_util.Format.DER
-        x509_req = x509.load_der_x509_csr(csr)
+    warnings.warn("get_names_from_req is deprecated and will be removed in the next "
+        "major release.", DeprecationWarning)
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", "acme.crypto_util.Format is deprecated")
+        if typ is None:
+            typ = acme_crypto_util.Format.PEM
+        typ = acme_crypto_util.Format(typ)
+        if typ == acme_crypto_util.Format.PEM:
+            x509_req = x509.load_pem_x509_csr(csr)
+        else:
+            assert typ == acme_crypto_util.Format.DER
+            x509_req = x509.load_der_x509_csr(csr)
     return acme_crypto_util.get_names_from_subject_and_extensions(
         x509_req.subject, x509_req.extensions
     )
@@ -517,7 +568,7 @@ CERT_PEM_REGEX = re.compile(
 )
 
 
-def cert_and_chain_from_fullchain(fullchain_pem: str) -> Tuple[str, str]:
+def cert_and_chain_from_fullchain(fullchain_pem: str) -> tuple[str, str]:
     """Split fullchain_pem into cert_pem and chain_pem
 
     :param str fullchain_pem: concatenated cert + chain
@@ -538,7 +589,7 @@ def cert_and_chain_from_fullchain(fullchain_pem: str) -> Tuple[str, str]:
 
     # Second pass: for each certificate found, parse it using cryptography and re-encode it,
     # with the effect of normalizing any encoding variations (e.g. CRLF, whitespace).
-    certs_normalized: List[str] = []
+    certs_normalized: list[str] = []
     for cert_pem in certs:
         cert = x509.load_pem_x509_certificate(cert_pem)
         cert_pem = cert.public_bytes(Encoding.PEM)
@@ -561,7 +612,7 @@ def get_serial_from_cert(cert_path: str) -> int:
     return cert.serial_number
 
 
-def find_chain_with_issuer(fullchains: List[str], issuer_cn: str,
+def find_chain_with_issuer(fullchains: list[str], issuer_cn: str,
                            warn_on_no_match: bool = False) -> str:
     """Chooses the first certificate chain from fullchains whose topmost
     intermediate has an Issuer Common Name matching issuer_cn (in other words
